@@ -1126,7 +1126,7 @@ static int check_comment_range(snd_ctl_t *handle, snd_config_t *conf,
 }
 
 static int restore_config_value(snd_ctl_t *handle, snd_ctl_elem_info_t *info,
-				snd_ctl_elem_iface_t type,
+				snd_ctl_elem_type_t type,
 				snd_config_t *value,
 				snd_ctl_elem_value_t *ctl, int idx,
 				int doit)
@@ -1175,7 +1175,7 @@ static int restore_config_value(snd_ctl_t *handle, snd_ctl_elem_info_t *info,
 }
 
 static int restore_config_value2(snd_ctl_t *handle, snd_ctl_elem_info_t *info,
-				 snd_ctl_elem_iface_t type,
+				 snd_ctl_elem_type_t type,
 				 snd_config_t *value,
 				 snd_ctl_elem_value_t *ctl, int idx,
 				 unsigned int numid, int doit)
@@ -1201,32 +1201,114 @@ static int restore_config_value2(snd_ctl_t *handle, snd_ctl_elem_info_t *info,
 	return 0;
 }
 
-static int set_control(snd_ctl_t *handle, snd_config_t *control,
-		       int *maxnumid, int doit)
+static int parse_config_id(snd_config_t *control, snd_ctl_elem_id_t *id,
+			   snd_config_t **comment, snd_config_t **value, int doit)
+{
+	snd_config_iterator_t i, next;
+	const char *numid;
+	snd_ctl_elem_iface_t iface = -1;
+	long device = -1;
+	long subdevice = -1;
+	const char *name = NULL;
+	long index = -1;
+
+	snd_ctl_elem_id_clear(id);
+	if (snd_config_get_id(control, &numid) < 0)
+		return -EINVAL;
+	snd_config_for_each(i, next, control) {
+		snd_config_t *n = snd_config_iterator_entry(i);
+		const char *fld;
+		if (snd_config_get_id(n, &fld) < 0)
+			continue;
+		if (strcmp(fld, "comment") == 0) {
+			if (snd_config_get_type(n) != SND_CONFIG_TYPE_COMPOUND) {
+				cerror(doit, "control.%s.%s is invalid", numid, fld);
+				return -EINVAL;
+			}
+			if (comment)
+				*comment = n;
+			continue;
+		}
+		if (strcmp(fld, "iface") == 0) {
+			iface = (snd_ctl_elem_iface_t)config_iface(n);
+			if (iface < 0)
+				return -EINVAL;
+			continue;
+		}
+		if (strcmp(fld, "device") == 0) {
+			if (snd_config_get_type(n) != SND_CONFIG_TYPE_INTEGER) {
+				cerror(doit, "control.%s.%s is invalid", numid, fld);
+				return -EINVAL;
+			}
+			if (snd_config_get_integer(n, &device) < 0)
+				return -EINVAL;
+			continue;
+		}
+		if (strcmp(fld, "subdevice") == 0) {
+			if (snd_config_get_type(n) != SND_CONFIG_TYPE_INTEGER) {
+				cerror(doit, "control.%s.%s is invalid", numid, fld);
+				return -EINVAL;
+			}
+			if (snd_config_get_integer(n, &subdevice) < 0)
+				return -EINVAL;
+			continue;
+		}
+		if (strcmp(fld, "name") == 0) {
+			if (snd_config_get_type(n) != SND_CONFIG_TYPE_STRING) {
+				cerror(doit, "control.%s.%s is invalid", numid, fld);
+				return -EINVAL;
+			}
+			if (snd_config_get_string(n, &name) < 0)
+				return -EINVAL;
+			continue;
+		}
+		if (strcmp(fld, "index") == 0) {
+			if (snd_config_get_type(n) != SND_CONFIG_TYPE_INTEGER) {
+				cerror(doit, "control.%s.%s is invalid", numid, fld);
+				return -EINVAL;
+			}
+			if (snd_config_get_integer(n, &index) < 0)
+				return -EINVAL;
+			continue;
+		}
+		if (strcmp(fld, "value") == 0) {
+			if (value)
+				*value = n;
+			continue;
+		}
+		cerror(doit, "unknown control.%s.%s field", numid, fld);
+	}
+	if (device < 0)
+		device = 0;
+	if (subdevice < 0)
+		subdevice = 0;
+	if (index < 0)
+		index = 0;
+	snd_ctl_elem_id_set_interface(id, iface);
+	snd_ctl_elem_id_set_device(id, device);
+	snd_ctl_elem_id_set_subdevice(id, subdevice);
+	snd_ctl_elem_id_set_name(id, name);
+	snd_ctl_elem_id_set_index(id, index);
+	return 0;
+}
+
+static int set_control(snd_ctl_t *handle, snd_config_t *control, int doit)
 {
 	snd_ctl_elem_value_t *ctl;
 	snd_ctl_elem_info_t *info;
 	snd_config_iterator_t i, next;
-	unsigned int numid1;
-	snd_ctl_elem_iface_t iface = -1;
-	int iface1;
-	const char *name1;
+	snd_ctl_elem_id_t *elem_id, *elem_id1;
 	unsigned int numid;
 	snd_ctl_elem_type_t type;
 	unsigned int count;
-	long device = -1;
-	long device1;
-	long subdevice = -1;
-	long subdevice1;
-	const char *name = NULL;
-	long index1;
-	long index = -1;
 	snd_config_t *value = NULL;
 	snd_config_t *comment = NULL;
 	unsigned int idx;
 	int err;
 	char *set;
 	const char *id;
+	snd_ctl_elem_id_alloca(&elem_id);
+	snd_ctl_elem_id_alloca(&elem_id1);
 	snd_ctl_elem_value_alloca(&ctl);
 	snd_ctl_elem_info_alloca(&info);
 	if (snd_config_get_type(control) != SND_CONFIG_TYPE_COMPOUND) {
@@ -1239,92 +1321,17 @@ static int set_control(snd_ctl_t *handle, snd_config_t *control,
 		return -EINVAL;
 	}
 	numid = atoi(id);
-	if ((int)numid > *maxnumid)
-		*maxnumid = numid;
-	snd_config_for_each(i, next, control) {
-		snd_config_t *n = snd_config_iterator_entry(i);
-		const char *fld;
-		if (snd_config_get_id(n, &fld) < 0)
-			continue;
-		if (strcmp(fld, "comment") == 0) {
-			if (snd_config_get_type(n) != SND_CONFIG_TYPE_COMPOUND) {
-				cerror(doit, "control.%d.%s is invalid", numid, fld);
-				return -EINVAL;
-			}
-			comment = n;
-			continue;
-		}
-		if (strcmp(fld, "iface") == 0) {
-			iface = (snd_ctl_elem_iface_t)config_iface(n);
-			if (iface < 0)
-				return -EINVAL;
-			continue;
-		}
-		if (strcmp(fld, "device") == 0) {
-			if (snd_config_get_type(n) != SND_CONFIG_TYPE_INTEGER) {
-				cerror(doit, "control.%d.%s is invalid", numid, fld);
-				return -EINVAL;
-			}
-			if (snd_config_get_integer(n, &device) < 0)
-				return -EINVAL;
-			continue;
-		}
-		if (strcmp(fld, "subdevice") == 0) {
-			if (snd_config_get_type(n) != SND_CONFIG_TYPE_INTEGER) {
-				cerror(doit, "control.%d.%s is invalid", numid, fld);
-				return -EINVAL;
-			}
-			if (snd_config_get_integer(n, &subdevice) < 0)
-				return -EINVAL;
-			continue;
-		}
-		if (strcmp(fld, "name") == 0) {
-			if (snd_config_get_type(n) != SND_CONFIG_TYPE_STRING) {
-				cerror(doit, "control.%d.%s is invalid", numid, fld);
-				return -EINVAL;
-			}
-			if (snd_config_get_string(n, &name) < 0)
-				return -EINVAL;
-			continue;
-		}
-		if (strcmp(fld, "index") == 0) {
-			if (snd_config_get_type(n) != SND_CONFIG_TYPE_INTEGER) {
-				cerror(doit, "control.%d.%s is invalid", numid, fld);
-				return -EINVAL;
-			}
-			if (snd_config_get_integer(n, &index) < 0)
-				return -EINVAL;
-			continue;
-		}
-		if (strcmp(fld, "value") == 0) {
-			value = n;
-			continue;
-		}
-		cerror(doit, "unknown control.%d.%s field", numid, fld);
-	}
-	if (!value) {
-		cerror(doit, "missing control.%d.value", numid);
-		return -EINVAL;
-	}
-	if (device < 0)
-		device = 0;
-	if (subdevice < 0)
-		subdevice = 0;
-	if (index < 0)
-		index = 0;
-
+	err = parse_config_id(control, elem_id, &comment, &value, doit);
+	if (err < 0)
+		return err;
 	err = -EINVAL;
 	if (!force_restore) {
 		snd_ctl_elem_info_set_numid(info, numid);
 		err = snd_ctl_elem_info(handle, info);
 	}
-	if (err < 0 && name) {
-		snd_ctl_elem_info_set_numid(info, 0);
-		snd_ctl_elem_info_set_interface(info, iface);
-		snd_ctl_elem_info_set_device(info, device);
-		snd_ctl_elem_info_set_subdevice(info, subdevice);
-		snd_ctl_elem_info_set_name(info, name);
-		snd_ctl_elem_info_set_index(info, index);
+	id = snd_ctl_elem_id_get_name(elem_id);
+	if (err < 0 && id && id[0]) {
+		snd_ctl_elem_info_set_id(info, elem_id);
 		err = snd_ctl_elem_info(handle, info);
 		if (err < 0 && comment && check_comment_access(comment, "user")) {
 			err = add_user_control(handle, info, comment);
@@ -1339,28 +1346,39 @@ static int set_control(snd_ctl_t *handle, snd_config_t *control,
 		cerror(doit, "failed to obtain info for control #%d (%s)", numid, snd_strerror(err));
 		return -ENOENT;
 	}
-	numid1 = snd_ctl_elem_info_get_numid(info);
-	iface1 = snd_ctl_elem_info_get_interface(info);
-	device1 = snd_ctl_elem_info_get_device(info);
-	subdevice1 = snd_ctl_elem_info_get_subdevice(info);
-	name1 = snd_ctl_elem_info_get_name(info);
-	index1 = snd_ctl_elem_info_get_index(info);
 	count = snd_ctl_elem_info_get_count(info);
+	snd_ctl_elem_info_get_id(info, elem_id1);
 	type = snd_ctl_elem_info_get_type(info);
-	if (err |= numid != numid1 && !force_restore)
-		cerror(doit, "warning: numid mismatch (%d/%d) for control #%d", 
-		      numid, numid1, numid);
-	if (err |= iface != iface1)
-		cerror(doit, "warning: iface mismatch (%d/%d) for control #%d", iface, iface1, numid);
-	if (err |= device != device1)
-		cerror(doit, "warning: device mismatch (%ld/%ld) for control #%d", device, device1, numid);
-	if (err |= subdevice != subdevice1)
-		cerror(doit, "warning: subdevice mismatch (%ld/%ld) for control #%d", subdevice, subdevice1, numid);
-	if (err |= strcmp(name, name1))
-		cerror(doit, "warning: name mismatch (%s/%s) for control #%d", name, name1, numid);
-	if (err |= index != index1)
-		cerror(doit, "warning: index mismatch (%ld/%ld) for control #%d", index, index1, numid);
-	if (err < 0) {
+	err = 0;
+	if (err |= !force_restore && numid != snd_ctl_elem_id_get_numid(elem_id1))
+		cerror(doit, "warning: numid mismatch (%u/%u) for control #%d",
+			     (unsigned int)numid, snd_ctl_elem_id_get_numid(elem_id1), numid);
+	if (err |= snd_ctl_elem_id_get_interface(elem_id) != snd_ctl_elem_id_get_interface(elem_id1))
+		cerror(doit, "warning: iface mismatch (%d/%d) for control #%d",
+			     (int)snd_ctl_elem_id_get_interface(elem_id),
+			     (int)snd_ctl_elem_id_get_interface(elem_id1),
+			     numid);
+	if (err |= snd_ctl_elem_id_get_device(elem_id) != snd_ctl_elem_id_get_device(elem_id1))
+		cerror(doit, "warning: device mismatch (%u/%u) for control #%d",
+			     snd_ctl_elem_id_get_device(elem_id),
+			     snd_ctl_elem_id_get_device(elem_id1),
+			     numid);
+	if (err |= snd_ctl_elem_id_get_subdevice(elem_id) != snd_ctl_elem_id_get_subdevice(elem_id1))
+		cerror(doit, "warning: subdevice mismatch (%u/%%u) for control #%d",
+			     snd_ctl_elem_id_get_subdevice(elem_id),
+			     snd_ctl_elem_id_get_subdevice(elem_id1),
+			     numid);
+	if (err |= strcmp(snd_ctl_elem_id_get_name(elem_id), snd_ctl_elem_id_get_name(elem_id1)))
+		cerror(doit, "warning: name mismatch (%s/%s) for control #%d",
+			     snd_ctl_elem_id_get_name(elem_id),
+			     snd_ctl_elem_id_get_name(elem_id1),
+			     numid);
+	if (err |= snd_ctl_elem_id_get_index(elem_id) != snd_ctl_elem_id_get_index(elem_id1))
+		cerror(doit, "warning: index mismatch (%u/%u) for control #%d",
+			     snd_ctl_elem_id_get_index(elem_id),
+			     snd_ctl_elem_id_get_index(elem_id1),
+			     numid);
+	if (err) {
 		cerror(doit, "failed to obtain info for control #%d (%s)", numid, snd_strerror(err));
 		return -ENOENT;
 	}
@@ -1383,7 +1401,7 @@ static int set_control(snd_ctl_t *handle, snd_config_t *control,
 	if (snd_ctl_elem_info_is_inactive(info) ||
 				!snd_ctl_elem_info_is_writable(info))
 		return 0;
-	snd_ctl_elem_value_set_numid(ctl, numid1);
+	snd_ctl_elem_value_set_numid(ctl, snd_ctl_elem_id_get_numid(elem_id1));
 
 	if (count == 1) {
 		err = restore_config_value(handle, info, type, value, ctl, 0, doit);
@@ -1472,8 +1490,14 @@ static int set_control(snd_ctl_t *handle, snd_config_t *control,
  _ok:
 	err = doit ? snd_ctl_elem_write(handle, ctl) : 0;
 	if (err < 0) {
-		error("Cannot write control '%d:%ld:%ld:%s:%ld' : %s", (int)iface, device, subdevice, name, index, snd_strerror(err));
+		char *s = snd_ctl_ascii_elem_id_get(elem_id1);
+		error("Cannot write control '%s' : %s", s, snd_strerror(err));
+		free(s);
 		return err;
+	} else if (debugflag) {
+		char *s = snd_ctl_ascii_elem_id_get(elem_id1);
+		dbg("control '%s' restored", s);
+		free(s);
 	}
 	return 0;
 }
@@ -1487,7 +1511,7 @@ static int set_controls(int card, snd_config_t *top, int doit)
 	snd_ctl_elem_id_t *elem_id;
 	snd_config_t *control;
 	snd_config_iterator_t i, next;
-	int err, maxnumid = -1, maxnumid2 = -1;
+	int err, controls1 = -1, controls2 = -1;
 	unsigned int idx, count = 0;
 	char name[32], tmpid[16];
 	const char *id;
@@ -1527,11 +1551,13 @@ static int set_controls(int card, snd_config_t *top, int doit)
 		cerror(doit, "state.%s.control is not a compound\n", id);
 		return -EINVAL;
 	}
+	controls1 = 0;
 	snd_config_for_each(i, next, control) {
 		snd_config_t *n = snd_config_iterator_entry(i);
-		err = set_control(handle, n, &maxnumid, doit);
+		err = set_control(handle, n, doit);
 		if (err < 0 && (!force_restore || !doit))
 			goto _close;
+		controls1++;
 	}
 
 	if (doit)
@@ -1555,7 +1581,7 @@ static int set_controls(int card, snd_config_t *top, int doit)
 		error("Cannot determine controls (2): %s", snd_strerror(err));
 		goto _free;
 	}
-	maxnumid2 = 0;
+	controls2 = 0;
 	/* skip non-readable elements */
 	for (idx = 0; idx < count; ++idx) {
 		snd_ctl_elem_info_clear(elem_info);
@@ -1564,23 +1590,23 @@ static int set_controls(int card, snd_config_t *top, int doit)
 		if (snd_ctl_elem_info(handle, elem_info) == 0) {
 			if (!snd_ctl_elem_info_is_readable(elem_info))
 				continue;
-			maxnumid2++;
+			controls2++;
 		}
 	}
 
 	/* check if we have additional controls in driver */
 	/* in this case we should go through init procedure */
  _check:
-	dbg("maxnumid=%i maxnumid2=%i", maxnumid, maxnumid2);
-	if (maxnumid >= 0 && maxnumid != maxnumid2) {
+	dbg("controls1=%i controls2=%i", controls1, controls2);
+	if (controls1 >= 0 && controls1 != controls2) {
 		/* not very informative */
 		/* but value is used for check only */
 		err = -EAGAIN;
-		dbg("more controls than maxnumid?");
+		dbg("config controls mismatch with hardware?");
 	}
 
  _free:
-	if (count >= 0)
+	if (count > 0)
 		snd_ctl_elem_list_free_space(list);
  _close:
 	snd_ctl_close(handle);
@@ -1614,7 +1640,7 @@ int save_state(const char *file, const char *cardname)
 		}
 		strcpy(nfile, file);
 		strcat(nfile, ".new");
-		lock_fd = state_lock(file, 10);
+		lock_fd = state_lock(file, LOCK_TIMEOUT);
 		if (lock_fd < 0) {
 			err = lock_fd;
 			goto out;
@@ -1675,7 +1701,7 @@ int load_state(const char *cfgdir, const char *file,
 	       const char *initfile, int initflags,
 	       const char *cardname, int do_init)
 {
-	int err, finalerr = 0, open_failed;
+	int err, finalerr = 0, open_failed, lock_fd;
 	struct snd_card_iterator iter;
 	snd_config_t *config;
 	const char *cardname1;
@@ -1695,7 +1721,14 @@ int load_state(const char *cfgdir, const char *file,
 		while ((cardname1 = snd_card_iterator_next(&iter)) != NULL) {
 			if (!do_init)
 				break;
+			lock_fd = card_lock(iter.card, LOCK_TIMEOUT);
+			if (lock_fd < 0) {
+				finalerr = lock_fd;
+				initfailed(iter.card, "lock", err);
+				continue;
+			}
 			err = init(cfgdir, initfile, initflags | FLAG_UCM_FBOOT | FLAG_UCM_BOOT, cardname1);
+			card_unlock(lock_fd, iter.card);
 			if (err < 0) {
 				finalerr = err;
 				initfailed(iter.card, "init", err);
@@ -1712,6 +1745,12 @@ int load_state(const char *cfgdir, const char *file,
 	if (err < 0)
 		goto out;
 	while ((cardname1 = snd_card_iterator_next(&iter)) != NULL) {
+		lock_fd = card_lock(iter.card, LOCK_TIMEOUT);
+		if (lock_fd < 0) {
+			initfailed(iter.card, "lock", lock_fd);
+			finalerr = lock_fd;
+			continue;
+		}
 		/* error is ignored */
 		init_ucm(initflags | FLAG_UCM_FBOOT, iter.card);
 		/* do a check if controls matches state file */
@@ -1727,6 +1766,7 @@ int load_state(const char *cfgdir, const char *file,
 				finalerr = err;
 			initfailed(iter.card, "restore", err);
 		}
+		card_unlock(lock_fd, iter.card);
 	}
 	err = finalerr ? finalerr : snd_card_iterator_error(&iter);
 out:
