@@ -30,6 +30,7 @@
 #include <syslog.h>
 #include <sys/stat.h>
 #include <sys/mman.h>
+#include <limits.h>
 #include "alsactl.h"
 
 int file_map(const char *filename, char **buf, size_t *bufsize)
@@ -210,7 +211,7 @@ int load_configuration(const char *file, snd_config_t **top, int *open_failed)
 	if (stdio_flag) {
 		err = snd_input_stdio_attach(&in, stdin, 0);
 	} else {
-		lock_fd = state_lock(file, 10);
+		lock_fd = state_lock(file, LOCK_TIMEOUT);
 		err = lock_fd >= 0 ? snd_input_stdio_open(&in, file, "r") : lock_fd;
 	}
 	if (err < 0) {
@@ -220,15 +221,17 @@ int load_configuration(const char *file, snd_config_t **top, int *open_failed)
 	}
 	err = snd_config_load(config, in);
 	snd_input_close(in);
-	if (lock_fd >= 0)
-		state_unlock(lock_fd, file);
 	if (err < 0) {
 		error("snd_config_load error: %s", snd_strerror(err));
 out:
+		if (lock_fd >= 0)
+			state_unlock(lock_fd, file);
 		snd_config_delete(config);
 		snd_config_update_free_global();
 		return err;
 	} else {
+		if (lock_fd >= 0)
+			state_unlock(lock_fd, file);
 		*top = config;
 		return 0;
 	}
@@ -247,6 +250,8 @@ int snd_card_iterator_sinit(struct snd_card_iterator *iter, const char *cardname
 	int cardno = -1;
 
 	if (cardname) {
+		if (strncmp(cardname, "hw:", 3) == 0)
+			cardname += 3;
 		cardno = snd_card_get_index(cardname);
 		if (cardno < 0) {
 			error("Cannot find soundcard '%s'...", cardname);
