@@ -58,7 +58,9 @@ enum uc_cmd {
 	/* set/get */
 	OM_SET,
 	OM_GET,
+	OM_GET_VAL,
 	OM_GETI,
+	OM_GETI_VAL,
 
 	/* misc */
 	OM_HELP,
@@ -82,7 +84,9 @@ static struct cmd cmds[] = {
 	{ OM_LIST2, 1, 1, "list" },
 	{ OM_SET, 2, 1, "set" },
 	{ OM_GET, 1, 1, "get" },
+	{ OM_GET_VAL, 1, 1, "getval" },
 	{ OM_GETI, 1, 1, "geti" },
+	{ OM_GETI_VAL, 1, 1, "getival" },
 	{ OM_DUMP, 1, 1, "dump" },
 	{ OM_HELP, 0, 0, "help" },
 	{ OM_QUIT, 0, 0, "quit" },
@@ -119,6 +123,20 @@ static void dump_help(struct context *context)
 );
 }
 
+int is_long(const char *str)
+{
+	char *end;
+	if (!*str)
+		return 0;
+	errno = 0;
+	strtol(str, &end, 10);
+	if (errno)
+		return 0;
+	if (*end)
+		return 0;
+	return 1;
+}
+
 static int parse_line(struct context *context, char *line)
 {
 	char *start, **nargv;
@@ -130,6 +148,8 @@ static int parse_line(struct context *context, char *line)
 							*line == '\n'))
 			line++;
 		c = *line;
+		if (c == '\0')
+			return 0;
 		if (c == '\"' || c == '\'') {
 			start = ++line;
 			while (*line && *line != c)
@@ -196,6 +216,15 @@ static void do_initial_open(struct context *context)
 
 	/* open library */
 	if (!context->no_open) {
+		if (is_long(context->card)) {
+			snprintf(name, sizeof(name), "hw:%s", context->card);
+			free(context->card);
+			context->card = strdup(name);
+		}
+		if (context->card == NULL) {
+			fprintf(stderr, "%s: empty card name\n", context->command);
+			my_exit(context, EXIT_FAILURE);
+		}
 		err = snd_use_case_mgr_open(&context->uc_mgr,
 					    context->card);
 		if (err < 0) {
@@ -326,6 +355,7 @@ static int do_one(struct context *context, struct cmd *cmd, char **argv)
 		}
 		break;
 	case OM_GET:
+	case OM_GET_VAL:
 		err = snd_use_case_get(context->uc_mgr, argv[0], &str);
 		if (err < 0) {
 			fprintf(stderr,
@@ -334,10 +364,14 @@ static int do_one(struct context *context, struct cmd *cmd, char **argv)
 				snd_strerror(err));
 			return err;
 		}
-		printf("  %s=%s\n", argv[0], str);
+		if (cmd->code == OM_GET)
+			printf("  %s=%s\n", argv[0], str);
+		else
+			printf("%s\n", str);
 		free((void *)str);
 		break;
 	case OM_GETI:
+	case OM_GETI_VAL:
 		err = snd_use_case_geti(context->uc_mgr, argv[0], &lval);
 		if (err < 0) {
 			fprintf(stderr,
@@ -346,7 +380,10 @@ static int do_one(struct context *context, struct cmd *cmd, char **argv)
 				snd_strerror(err));
 			return lval;
 		}
-		printf("  %s=%li\n", argv[0], lval);
+		if (cmd->code == OM_GETI)
+			printf("  %s=%li\n", argv[0], lval);
+		else
+			printf("%li\n", lval);
 		break;
 	case OM_QUIT:
 		context->do_exit = 1;
@@ -473,7 +510,7 @@ int main(int argc, char *argv[])
 			in = fopen(context->batch, "r");
 			if (in == NULL) {
 				fprintf(stderr, "%s: error failed to open file '%s': %s\n",
-					command, context->batch, strerror(-errno));
+					command, context->batch, strerror(errno));
 				my_exit(context, EXIT_FAILURE);
 			}
 		}
