@@ -34,7 +34,7 @@ struct select_state {
 	fd_set rfds_ex;
 };
 
-static int select_prepare(struct waiter_context *waiter)
+static int select_prepare(struct waiter_context *waiter ATTRIBUTE_UNUSED)
 {
 	return 0;
 }
@@ -53,7 +53,7 @@ static int select_wait_event(struct waiter_context *waiter, int timeout_msec)
 	FD_ZERO(&state->rfds_ex);
 
 	fd_max = 0;
-	for (i = 0; i < waiter->pfd_count; ++i) {
+	for (i = 0; i < (int)waiter->pfd_count; ++i) {
 		pfd = &waiter->pfds[i];
 
 		if (pfd->events & POLLIN_SET)
@@ -77,9 +77,9 @@ static int select_wait_event(struct waiter_context *waiter, int timeout_msec)
 	err = select(fd_max + 1, &state->rfds_rd, &state->rfds_wr,
 		     &state->rfds_ex, tv_ptr);
 	if (err < 0)
-		return err;
+		return -errno;
 
-	for (i = 0; i < waiter->pfd_count; ++i) {
+	for (i = 0; i < (int)waiter->pfd_count; ++i) {
 		pfd = &waiter->pfds[i];
 
 		pfd->revents = 0;
@@ -91,10 +91,10 @@ static int select_wait_event(struct waiter_context *waiter, int timeout_msec)
 			pfd->revents |= POLLHUP;
 	}
 
-	return 0;
+	return err;
 }
 
-static void select_release(struct waiter_context *waiter)
+static void select_release(struct waiter_context *waiter ATTRIBUTE_UNUSED)
 {
 	return;
 }
