@@ -27,7 +27,7 @@
 #include <string.h>
 #include <signal.h>
 #include <getopt.h>
-#include <sys/poll.h>
+#include <poll.h>
 #include <alsa/asoundlib.h>
 #include "aconfig.h"
 #include "version.h"
@@ -455,7 +455,7 @@ static void record_port_numbers(void)
 
 static void record_event(const snd_seq_event_t *ev)
 {
-	unsigned int i;
+	int i;
 	struct smf_track *track;
 
 	/* ignore events without proper timestamps */
@@ -583,7 +583,7 @@ static void record_event(const snd_seq_event_t *ev)
 		else
 			command(track, 0xf7), i = 0;
 		var_value(track, ev->data.ext.len - i);
-		for (; i < ev->data.ext.len; ++i)
+		for (; i < (int)ev->data.ext.len; ++i)
 			add_byte(track, ((unsigned char*)ev->data.ext.ptr)[i]);
 		break;
 	default:
@@ -719,7 +719,7 @@ static void version(void)
 	fputs("arecordmidi version " SND_UTIL_VERSION_STR "\n", stderr);
 }
 
-static void sighandler(int sig)
+static void sighandler(int sig ATTRIBUTE_UNUSED)
 {
 	stop = 1;
 }
@@ -740,7 +740,7 @@ int main(int argc, char *argv[])
 		{"metronome", 1, NULL, 'm'},
 		{"timesig", 1, NULL, 'i'},
 		{"num-events", 1, NULL, 'n'},
-		{ }
+		{0}
 	};
 
 	char *filename = NULL;
