@@ -17,6 +17,7 @@
   The full GNU General Public License is included in this distribution
   in the file called LICENSE.GPL.
 */
+#include "aconfig.h"
 #include <assert.h>
 #include <errno.h>
 #include <stddef.h>
@@ -30,7 +31,8 @@
 
 /* Parse VendorToken object, create the "SectionVendorToken" and save it */
 int tplg_build_vendor_token_object(struct tplg_pre_processor *tplg_pp,
-				   snd_config_t *obj_cfg, snd_config_t *parent)
+				   snd_config_t *obj_cfg,
+				   snd_config_t *parent ATTRIBUTE_UNUSED)
 {
 	snd_config_iterator_t i, next;
 	snd_config_t *vtop, *n, *obj;
@@ -115,7 +117,7 @@ int tplg_parent_update(struct tplg_pre_processor *tplg_pp, snd_config_t *parent,
 		return ret;
 
 	/* get section config */
-	if (!strcmp(section_name, "tlv")) {
+	if (!strcmp(section_name, "tlv") || !strcmp(section_name, "texts")) {
 		/* set tlv name if config exists already */
 		ret = snd_config_search(cfg, section_name, &item_config);
 			if (ret < 0) {
@@ -195,7 +197,7 @@ int tplg_build_data_object(struct tplg_pre_processor *tplg_pp, snd_config_t *obj
 	return tplg_parent_update(tplg_pp, parent, "data", name);
 }
 
-static int tplg_create_config_template(struct tplg_pre_processor *tplg_pp,
+static int tplg_create_config_template(struct tplg_pre_processor *tplg_pp ATTRIBUTE_UNUSED,
 				       snd_config_t **template,
 				       const struct config_template_items *items)
 {
@@ -498,7 +500,7 @@ min_max_check:
 }
 
 /* get object's name attribute value */
-const char *tplg_object_get_name(struct tplg_pre_processor *tplg_pp,
+const char *tplg_object_get_name(struct tplg_pre_processor *tplg_pp ATTRIBUTE_UNUSED,
 				 snd_config_t *object)
 {
 	snd_config_t *cfg;
@@ -517,7 +519,7 @@ const char *tplg_object_get_name(struct tplg_pre_processor *tplg_pp,
 }
 
 /* look up the instance of object in a config */
-static snd_config_t *tplg_object_lookup_in_config(struct tplg_pre_processor *tplg_pp,
+static snd_config_t *tplg_object_lookup_in_config(struct tplg_pre_processor *tplg_pp ATTRIBUTE_UNUSED,
 						  snd_config_t *class, const char *type,
 						  const char *class_name, const char *id)
 {
@@ -976,7 +978,7 @@ template:
 }
 
 static int tplg_build_generic_object(struct tplg_pre_processor *tplg_pp, snd_config_t *obj_cfg,
-				     snd_config_t *parent)
+				     snd_config_t *parent ATTRIBUTE_UNUSED)
 {
 	snd_config_t *wtop;
 	const char *name;
@@ -1036,6 +1038,15 @@ const struct config_template_items bytes_control_config = {
 	.compound_config_ids = {"access"}
 };
 
+const struct config_template_items enum_control_config = {
+	.int_config_ids = {"index"},
+	.compound_config_ids = {"access"}
+};
+
+const struct config_template_items text_config = {
+	.compound_config_ids = {"values"}
+};
+
 const struct config_template_items scale_config = {
 	.int_config_ids = {"min", "step", "mute"},
 };
@@ -1071,6 +1082,7 @@ const struct build_function_map object_build_map[] = {
 	{"Base", "ops", "ops" ,&tplg_build_ops_object, NULL, &ops_config},
 	{"Base", "extops", "extops" ,&tplg_build_ops_object, NULL, &ops_config},
 	{"Base", "channel", "channel", &tplg_build_channel_object, NULL, &channel_config},
+	{"Base", "text", "SectionText", &tplg_build_text_object, NULL, &text_config},
 	{"Base", "VendorToken", "SectionVendorTokens", &tplg_build_vendor_token_object,
 	 NULL, NULL},
 	{"Base", "hw_config", "SectionHWConfig", &tplg_build_hw_cfg_object, NULL,
@@ -1083,13 +1095,15 @@ const struct build_function_map object_build_map[] = {
 	 &mixer_control_config},
 	{"Control", "bytes", "SectionControlBytes", &tplg_build_bytes_control, NULL,
 	 &bytes_control_config},
+	 {"Control", "enum", "SectionControlEnum", &tplg_build_enum_control, NULL,
+	 &enum_control_config},
 	{"Dai", "", "SectionBE", &tplg_build_generic_object, NULL, &be_dai_config},
 	{"PCM", "pcm", "SectionPCM", &tplg_build_generic_object, NULL, &pcm_config},
 	{"PCM", "pcm_caps", "SectionPCMCapabilities", &tplg_build_pcm_caps_object,
 	 NULL, &pcm_caps_config},
 };
 
-static const struct build_function_map *tplg_object_get_map(struct tplg_pre_processor *tplg_pp,
+static const struct build_function_map *tplg_object_get_map(struct tplg_pre_processor *tplg_pp ATTRIBUTE_UNUSED,
 							    snd_config_t *obj)
 {
 	snd_config_iterator_t first;
@@ -1144,7 +1158,7 @@ snd_config_t *tplg_object_get_section(struct tplg_pre_processor *tplg_pp, snd_co
 }
 
 /* return 1 if attribute not found in search_config, 0 on success and negative value on error */
-static int tplg_object_copy_and_add_param(struct tplg_pre_processor *tplg_pp,
+static int tplg_object_copy_and_add_param(struct tplg_pre_processor *tplg_pp ATTRIBUTE_UNUSED,
 					  snd_config_t *obj,
 					  snd_config_t *attr_cfg,
 					  snd_config_t *search_config)
@@ -1350,8 +1364,8 @@ static int tplg_object_pre_process_children(struct tplg_pre_processor *tplg_pp,
 	return 0;
 }
 
-static int tplg_construct_object_name(struct tplg_pre_processor *tplg_pp, snd_config_t *obj,
-				      snd_config_t *class_cfg)
+static int tplg_construct_object_name(struct tplg_pre_processor *tplg_pp ATTRIBUTE_UNUSED,
+				      snd_config_t *obj, snd_config_t *class_cfg)
 {
 	snd_config_iterator_t i, next;
 	snd_config_t *args, *n;
@@ -1546,7 +1560,7 @@ static int tplg_object_set_unique_attribute(struct tplg_pre_processor *tplg_pp,
  * Helper function to get object instance config which is 2 nodes down from class_type config.
  * ex: Get the pointer to the config node with ID "0" from the input config Widget.pga.0 {}
  */
-snd_config_t *tplg_object_get_instance_config(struct tplg_pre_processor *tplg_pp,
+snd_config_t *tplg_object_get_instance_config(struct tplg_pre_processor *tplg_pp ATTRIBUTE_UNUSED,
 					snd_config_t *class_type)
 {
 	snd_config_iterator_t first;
@@ -1589,6 +1603,7 @@ pre_process_object_variables_expand_fcn(snd_config_t **dst, const char *str, voi
 	snd_config_t *object_cfg = tplg_pp->current_obj_cfg;
 	snd_config_t *conf_defines;
 	const char *object_id;
+	const char *val;
 	int ret;
 
 	ret = snd_config_search(tplg_pp->input_cfg, "Define", &conf_defines);
@@ -1600,14 +1615,30 @@ pre_process_object_variables_expand_fcn(snd_config_t **dst, const char *str, voi
 	if (ret >= 0)
 		return ret;
 
+	/* No global define found, proceeed to object attribute search */
 	if (snd_config_get_id(object_cfg, &object_id) < 0)
 		return -EINVAL;
 
 	/* find variable from object attribute values if not found in global definitions */
 	ret = pre_process_find_variable(dst, str, object_cfg);
-	if (ret < 0)
+	if (ret < 0) {
 		SNDERR("Failed to find definition for attribute %s in '%s' object\n",
 		       str, object_id);
+		return ret;
+	}
+
+	/* the extracted value may contain a nested $-expression */
+	if (snd_config_get_string(*dst, &val) >= 0) {
+		if (val[0] == '$') {
+			char *var = strdup(val);
+
+			snd_config_delete(*dst);
+			ret = snd_config_evaluate_string(dst, var,
+							 pre_process_object_variables_expand_fcn,
+							 tplg_pp);
+			free(var);
+		}
+	}
 
 	return ret;
 }
@@ -1623,7 +1654,7 @@ pre_process_object_variables_expand_fcn(snd_config_t **dst, const char *str, voi
  * or '\0'.
  *
  * In '$[<contents>]' case all letters but '[' and ']' are allow in
- * any sequence. Nested '[]' is also allowed if the number if '[' and
+ * any sequence. Nested '[]' is also allowed if the number of '[' and
  * ']' match.
  *
  * The function modifies *stringp, and *prefix - if not NULL - points
