@@ -20,6 +20,7 @@
   in the file called LICENSE.GPL.
 */
 
+#include "aconfig.h"
 #include <stdbool.h>
 #include <stdlib.h>
 #include <stdio.h>
@@ -35,6 +36,9 @@
 #include <alsa/asoundlib.h>
 #include <alsa/topology.h>
 #include "gettext.h"
+#ifdef ENABLE_NLS
+#include <locale.h>
+#endif
 #include "version.h"
 #include "topology.h"
 
@@ -92,7 +96,7 @@ static int load(const char *source_file, void **dst, size_t *dst_size)
 		fd = open(source_file, O_RDONLY);
 		if (fd < 0) {
 			fprintf(stderr, _("Unable to open input file '%s': %s\n"),
-				source_file, strerror(-errno));
+				source_file, strerror(errno));
 			return 1;
 		}
 	}
@@ -116,7 +120,7 @@ static int load(const char *source_file, void **dst, size_t *dst_size)
 		buf = buf2;
 	}
 	if (r < 0) {
-		fprintf(stderr, _("Read error: %s\n"), strerror(-errno));
+		fprintf(stderr, _("Read error: %s\n"), strerror(errno));
 		goto _err;
 	}
 
@@ -173,7 +177,7 @@ static int save(const char *output_file, void *buf, size_t size)
 		fd = open(fname, O_RDWR | O_CREAT | O_TRUNC, S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP | S_IROTH | S_IWOTH);
 		if (fd < 0) {
 			fprintf(stderr, _("Unable to open output file '%s': %s\n"),
-				fname, strerror(-errno));
+				fname, strerror(errno));
 			return 1;
 		}
 	}
@@ -190,11 +194,11 @@ static int save(const char *output_file, void *buf, size_t size)
 	}
 
 	if (r < 0) {
-		fprintf(stderr, _("Write error: %s\n"), strerror(-errno));
+		fprintf(stderr, _("Write error: %s\n"), strerror(errno));
 		if (fd != fileno(stdout)) {
 			if (fname && remove(fname))
 				fprintf(stderr, _("Unable to remove file %s: %s\n"),
-						fname, strerror(-errno));
+						fname, strerror(errno));
 			close(fd);
 		}
 		return 1;
@@ -205,7 +209,7 @@ static int save(const char *output_file, void *buf, size_t size)
 
 	if (fname && rename(fname, output_file)) {
 		fprintf(stderr, _("Unable to rename file '%s' to '%s': %s\n"),
-			fname, output_file, strerror(-errno));
+			fname, output_file, strerror(errno));
 		return 1;
 	}
 
@@ -394,9 +398,12 @@ static int add_define(char **defs, char *d)
 	size_t len = (*defs ? strlen(*defs) : 0) + strlen(d) + 2;
 	char *m = realloc(*defs, len);
 	if (m) {
-		if (*defs)
+		if (*defs) {
 			strcat(m, ",");
-		strcat(m, d);
+			strcat(m, d);
+		} else {
+			strcpy(m, d);
+		}
 		*defs = m;
 		return 0;
 	}
