@@ -25,11 +25,15 @@ struct tplg_pre_processor {
 	snd_config_t *output_cfg;
 	snd_output_t *output;
 	snd_output_t *dbg_output;
+	snd_config_t *current_obj_cfg;
+	snd_config_t *define_cfg;
+	snd_config_t *define_cfg_merged;
+	char *inc_path;
 };
 
 int pre_process(struct tplg_pre_processor *tplg_pp, char *config, size_t config_size,
 		const char *pre_processor_defs, const char *inc_path);
 int init_pre_processor(struct tplg_pre_processor **tplg_pp, snd_output_type_t type,
 		       const char *output_file);
-void free_pre_preprocessor(struct tplg_pre_processor *tplg_pp);
+void free_pre_processor(struct tplg_pre_processor *tplg_pp);
 #endif
